import json
from web3 import Web3, HTTPProvider
#from web3.auto.infura import w3

class contract_interface:
	def __init__(self):
		# truffle development blockchain address
		blockchain_address = 'https://kovan.infura.io/v3/61464b27cc9d42b2b889a1e8a71a952e'
		# Client instance to interact with the blockchain
		self.web3 = Web3(HTTPProvider(blockchain_address))

		# Set the default account (so we don't need to set the "from" for every transaction call)
		self.addr = ''#Specify your Ethereum public adress
		self.private_key = ''#Specify the associated private key

		self.web3.eth.defaultAddress = self.addr

		# Path to the compiled contract JSON file
		compiled_contract_path = './contract/build/contracts/PKI.json'
		# Deployed contract address (see `migrate` command output: `contract address`)
		deployed_contract_address = '0x2ddE419F1369a4eF6B6408245479589CAB4Ff974'

		with open(compiled_contract_path) as file:
			contract_json = json.load(file)  # load contract info as JSON
			contract_abi = contract_json['abi']  # fetch contract's abi - necessary to call its functions

		# Fetch deployed contract reference
		self.contract = self.web3.eth.contract(address=deployed_contract_address, abi=contract_abi)

	def lookup(self, uuid):
		tx = self.contract.functions.lookup(uuid).call()
		return tx
		
	def register(self, uuid, hash):
		tx = self.contract.functions.register(uuid, hash).buildTransaction({'nonce': self.web3.eth.getTransactionCount(self.addr), 'gas': 3000000})
		signed_tx = self.web3.eth.account.signTransaction(tx, private_key=self.private_key)
		self.web3.eth.sendRawTransaction(signed_tx.rawTransaction)
		return 0
		
	def update(self, uuid, hash):
		tx = self.contract.functions.update(uuid, hash).buildTransaction({'nonce': self.web3.eth.getTransactionCount(self.addr), 'gas': 3000000})
		signed_tx = self.web3.eth.account.signTransaction(tx, private_key=self.private_key)
		self.web3.eth.sendRawTransaction(signed_tx.rawTransaction)
		return 0

	def revoke(self):
		tx = self.contract.functions.revoke(uuid).buildTransaction({'nonce': self.web3.eth.getTransactionCount(self.addr), 'gas': 3000000})
		signed_tx = self.web3.eth.account.signTransaction(tx, private_key=self.private_key)
		self.web3.eth.sendRawTransaction(signed_tx.rawTransaction)
		return 0




