from http.server import HTTPServer, BaseHTTPRequestHandler
import ssl
import socket
from OpenSSL import crypto
import sys
from contract_interface import contract_interface
import os.path
import hashlib
import re
import os


def hash_file(filename):
   """"This function returns the SHA-1 hash
   of the file passed into it"""

   # make a hash object
   h = hashlib.sha1()

   # open file for reading in binary mode
   with open(filename,'rb') as file:

       # loop till the end of the file
       chunk = 0
       while chunk != b'':
           # read only 1024 bytes at a time
           chunk = file.read(1024)
           h.update(chunk)

   # return the hex representation of digest
   return h.hexdigest()

def sanitize(body):
	return str(body)[2:-1].replace('\\n', '\n')

def parse_contID(s):
	pattern = "1.2.3.4=(.*?)\'>"
	substring = re.search(pattern, s).group(1)
	return substring

def register_device(pem_data):
    update = False
    
    #print(pem_data)
    cert = crypto.load_certificate(crypto.FILETYPE_PEM, pem_data)
    device_uuid = str(cert.get_subject().CN)
    print("Received cert from: " + str(device_uuid))

    if parse_contID(str(cert.get_subject())) != "1234":
    	print("Certificate not valid. Controller UUID should be 1234")
    	return

    f= open("tmp.pem","w+")
    f.write("%s" % pem_data)
    f.close()
    
    filename = "./device_certs/" + device_uuid + ".pem"
    if(os.path.isfile(filename)):
        update = True
        a = os.popen("openssl verify -no-CApath -partial_chain -verbose -trusted " + filename + " tmp.pem | grep \"OK\"").read()
        #print("verify output: " + str(a))
        if a == "":
          print("Unauthorized certificate")
          os.system("rm tmp.pem")
          return

    os.system("rm tmp.pem")
    f= open(filename,"w+")
    f.write("%s" % pem_data)
    f.close()
    #while True:
    #  pass

    hash = hash_file(filename)
    contract = contract_interface()
    if update:
        contract.update(int(device_uuid), str(hash))
        print("Cert updated in contract\n")
    else:
        print(contract.register(int(device_uuid), str(hash)))
        print("Cert registered in contract\n")


class HandleRequests(BaseHTTPRequestHandler):
    def _set_headers(self):
        self.send_response(200)
        self.send_header('Content-type', 'text/html')
        self.end_headers()

    def do_GET(self):
        self._set_headers()
        print("Lookup request received for uuid: " + str(self.path[1:]))
        contract = contract_interface()
        self.wfile.write(str(contract.lookup(int(self.path[1:]))).encode("utf-8"))
        
    def do_POST(self):
        '''Reads post request body'''
        self._set_headers()
        content_len = int(self.headers.get('content-length', 0))
        post_body = self.rfile.read(content_len)
        pem_data = sanitize(post_body)
        register_device(pem_data)
        #self.wfile.write("received post request:<br>{}".format(post_body))

    def do_PUT(self):
        self.do_POST()


if len(sys.argv) == 2:
    # Get "IP address of Server" and also the "port number" from argument 1 and argument 2
    addr = sys.argv[1]
else:
    print("Run like : python controller.py <arg1:server ip>")
    exit(1)

httpd = HTTPServer((str(addr), 4443), HandleRequests)

httpd.socket = ssl.wrap_socket (httpd.socket, 
        keyfile="./key.pem", 
        certfile="./cert.pem", server_side=True)

httpd.serve_forever() 
